function [W,mu,Ln,niter,diff,LM] = EM_PLd(X,k1,W,V,mu)
% [W,M] = EM_hmm(X,k)
%
% EM algorithm for k Gaussian mixture estimation
%
% Inputs:
%   X(n) - input data, n=number of observations
%   k - maximum number of Gaussian components allowed

% Ouputs:
%   W(k) - estimated weights of GM
%   M(k) - estimated mean vectors of GM

%%%%%%%%%%%%%%%%%%%%%%
%%%% begin of EM_GM %%%%
%%%%%%%%%%%%%%%%%%%%%%
%%%% Initialize W, M, V,L %%%%


V =V;
k1=k1;
diff=1;
ltol = 0.00000001;
%%%% EM algorithm %%%%
niter = 0;
maxiter=1000;

while  (diff>ltol)&(niter<=maxiter),

    niter = niter + 1;
    Z = Expectation(X,k1,W,mu,V); % E-step
    [W,mu] = Maximization(X,k1,Z) ;% M-step
    Ln = Likelihood(X,k1,W,mu,V);
    LM(niter)=Ln;
    if niter>1
        diff=abs(Ln-L0);
    end
    L0 =Ln;

end

%%%%%%%%%%%%%%%%%%%%%%
%%%% End of EM_GM %%%%
%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Begin of Expectation %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Z = Expectation(X,k1,W,mu,V)
n = length(X);
Z = zeros(n,k1);
V=V;
for i=1:n,
    for k=1:k1,
        pl =( 1/(sqrt(2*pi)*sqrt(V(k))))*exp(-((X(i)-mu(k))^2)/(2*V(k)));
        Z(i,k) = W(k)*pl;
    end
    Z(i,:) = Z(i,:)/sum(Z(i,:));
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% End of Expectation %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% begin of Maximization %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [W,mu] = Maximization(X,k1,Z)
n = length(X);
W = zeros(1,k1); mu = zeros(1,k1);
for k=1:k1,  % Compute weights
    for i=1:n,
        W(k) = W(k) + Z(i,k);
    end
end
W = W/n;

u11=0;
for i=1:n
    for k=1:k1
        u11=u11+Z(i,k)*(X(i)-(k-1)*10);
    end
end
mu(1)=u11/n;

for k=1:k1
    mu(k)=mu(1)+(k-1)*10;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Begin of Likelihood %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%

function Lsum = Likelihood(X,k1,W,mu,V)
n = length(X);
for i=1:n,
    L=[ ];
    for k=1:k1
        L(k)=(exp(-(X(i)-mu(k))^2)/(2*V(k)))*W(k);
    end

    logLi(i)=log(sum(L));
end

Lsum=sum(logLi);
%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% End of Likelihood %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%










