function [W,mu,sig,Ln,niter,diff,LM,d] = EM_PLd_d(X,k1,W,s,mu,d)

% [W,M] = EM_hmm(X,k)
%
% EM algorithm for k Gausigsian mixture estimation
%
% Inputs:
%   X(n) - input data, n=number of observations
%   k - maximum number of Gaussian components allowed

% Ouputs:
%   W(k) - estimated weights of GM
%   M(k) - estimated mean vectors of GM

sig=s;
V =ones(1,length(W))*sig;
k1=k1;
diff=1;
ltol = 0.00000001;
%%%% EM algorithm %%%%

niter = 0;
maxiter=1000;

while  (diff>ltol)&(niter<=maxiter),
    niter = niter + 1;
    Z = Expectation(X,k1,W,mu,V); % E-step
    [W,mu,sig] = Maximization(X,k1,Z,sig,mu) ;% M-step
    V=(sig^2)*ones(1,k1);
    Ln = Likelihood(X,k1,W,mu,V);
    LM(niter)=Ln;
    if niter>1
        diff=abs(Ln-L0);
    end
    L0 =Ln;
end
%%%%%%%%%%%%%%%%%%%%%%
%%%% End of EM_GM %%%%
%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Begin of Expectation %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function Z = Expectation(X,k1,W,mu,V)
        n = length(X);
        Z = zeros(n,k1);
        sum_k=zeros(1,k1)
        for k=1:k1
            for j=round(mu(k)-20):round(mu(k)+20)
                sum_k(k)=sum_k(k)+( 1/(sqrt(2*pi)*sqrt(V(k))))*exp(-((j-mu(k))^2)/(2*V(k)));
            end
        end
        for i=1:n,
            for k=1:k1,
                pl =( 1/(sqrt(2*pi)*sqrt(V(k))))*exp(-((X(i)-mu(k))^2)/(2*V(k)))/sum_k(k);
                Z(i,k) = W(k)*pl;
            end
            Z(i,:) = Z(i,:)/sum(Z(i,:));
        end
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% End of Expectation %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% begin of Maximization %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%maximize over weights, mu and variance

    function [W,mu,sig] = Maximization(X,k1,Z,sig,mu)
        n = length(X);
        W = zeros(1,k1);
        for k=1:k1,  % Compute weights
            for i=1:n,
                W(k) = W(k) + Z(i,k);
            end
        end
        W = W/n;

        u11=mu(1);
        if(sig>1)
            u11=0;
            for i=1:n
                for k=1:k1
                    u11=u11+Z(i,k)*(X(i)-(k-1)*d);
                end
            end
            u11=u11/n;
            mu(1)=u11;

            %mle for d
            dnum=0;
            dden=0;
            for i=1:n
                for k=1:k1
                    dnum=dnum+Z(i,k)*(k-1)*(X(i)-u11);
                    dden=dden+Z(i,k)*(k-1)^2;
                end
            end
            d=dnum/dden;
            for k=1:k1
                mu(k)=mu(1)+(k-1)*d;
            end

            %mle of variance
            sig=0;
            for i=1:n
                for k=1:k1
                    sig=sig+Z(i,k)*(X(i)-mu(k))^2;
                end
            end
            sig=(sig/n)^0.5;
        end

        if sig<=1
            %%bisection for u11
            u11u=u11+5;
            u11l=u11-5;
            der=0.5;
            ite=1;
            while ((abs(der)>0.00001)&(ite<5000))
                ite=ite+1;
                %Second part
                der=0;
                firstPart=0;
                secondPart=0;
                for k=1:length(V)
                    num=0;
                    den=0;
                    for j=round(u11+(k-1)*d-20):round(u11+(k-1)*d+20)
                        num=num+exp(-(j-u11-(k-1)*d)^2/2/V(k))*(j-u11-(k-1)*d);
                        den=den+exp(-(j-u11-(k-1)*d)^2/2/V(k));
                    end
                    for i=1:length(X)
                        firstPart=firstPart+Z(i,k)*(X(i)-u11-(k-1)*d);
                        secondPart=secondPart+Z(i,k)*num/den;
                    end
                end
                der=firstPart-secondPart
                if abs(der)>0.00001
                    if der>0
                        u11l=u11;
                        u11= (u11+u11u)/2;
                    else
                        u11u=u11;
                        u11=(u11+u11l)/2;
                    end
                end
                if (ite==50000)&(der>0.0001)
                    u11=mu(1);
                end
            end

            %%bisection for d
            du=d+5;
            dl=d-5;
            der=0.5;
            ite=1;
            while ((abs(der)>0.00001)&(ite<50000))
                %Second part
                ite=ite+1;

                %Second part
                der=0;
                firstPart=0;
                secondPart=0;
                for k=1:length(V)
                    num=0;
                    den=0;
                    for j=round(u11+(k-1)*d-20):round(u11+(k-1)*d+20)
                        num=num+exp(-(j-u11-(k-1)*d)^2/2/V(k))*(j-u11-(k-1)*d)*(k-1);
                        den=den+exp(-(j-u11-(k-1)*d)^2/2/V(k));
                    end
                    for i=1:length(X)
                        firstPart=firstPart+Z(i,k)*(X(i)-u11-(k-1)*d)*(k-1);
                        secondPart=secondPart+Z(i,k)*num/den;
                    end
                end
                der=firstPart-secondPart
                if abs(der)>0.00001
                    if der>0
                        dl=d;
                        d= (d+du)/2;
                    else
                        du=d;
                        d=(d+dl)/2;
                    end
                end
                if (ite==50000)&(der>0.0001)
                    d=mu(2)-mu(1);
                end
                [d der];
            end

            for k=1:k1
                mu(k)=u11+(k-1)*d;
            end

            %%bisection for s

            sigU=sig+30;
            sigL=sig*0.1;
            der=0.5;
            ite=ite+1;
            while ((abs(der)>0.00001)&(ite<50000))
                ite=ite+1;
                %second part
                firstPart=0;
                secondPart=0;
                for k=1:length(V)
                    num=0;
                    den=0;
                    for j=round(u11+(k-1)*d-20):round(u11+(k-1)*d+20)
                        num=num+exp(-(j-u11-(k-1)*d)^2/2/sig^2)*(j-u11-(k-1)*d)^2/2/sig-0.5*exp(-(j-u11-(k-1)*d)^2/2/sig^2);
                        den=den+exp(-(j-u11-(k-1)*d)^2/2/sig^2);
                    end
                    for i=1:length(X)
                        firstPart=firstPart+Z(i,k)*(-0.5+(X(i)-u11-(k-1)*d)^2/2/sig^2);
                        secondPart=secondPart+Z(i,k)*num/den;
                    end
                end
                der=firstPart-secondPart;
                if(abs(der)>0.00001)
                    if der>0
                        sigL=sig;
                        sig= (sig+sigU)/2;
                    else
                        sigU=sig;
                        sig=(sig+sigL)/2;
                    end
                end
            end
            if (ite==50000)&(der>0.0001)
                sig=V(k)^0.5;
            end
        end
    end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Begin of Likelihood %%
%%%%%%%%%%%%%%%%%%%%%%%%%%%

    function Lsum = Likelihood(X,k1,W,mu,V)
        n = length(X);
        sum_k=zeros(1,k1)
        for k=1:k1
            for j=round(mu(k)-20):round(mu(k)+20)
                sum_k(k)=sum_k(k)+( 1/(sqrt(2*pi)*sqrt(V(k))))*exp(-((j-mu(k))^2)/(2*V(k)));
            end
        end
        for i=1:n,
            L=[ ];
            for k=1:k1
                L(k)=(1/(sqrt(2*pi)*sqrt(V(k))))*exp(-(X(i)-mu(k))^2/(2*V(k)))*W(k)/sum_k(k);
            end
            logLi(i)=log(sum(L));
        end
        Lsum=sum(logLi);
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        %%%% End of Likelihood %%%%
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
    end
end





